package tech.nevets.jaml.util;

public class SemanticVersioner implements Comparable<SemanticVersioner> {
    private String version;

    public SemanticVersioner(String version) {
        if (version == null) {
            throw new IllegalArgumentException("Version cannot be null");
        }

        if (!version.matches("[0-9]+(\\.[0-9]+)*")) {
            throw new IllegalArgumentException("Invalid version format");
        }

        this.version = version;
    }

    public final String get() {
        return this.version;
    }

    @Override
    public int compareTo(SemanticVersioner sv) {
        if(sv == null)
            return 1;
        String[] thisParts = this.get().split("\\.");
        String[] svParts = sv.get().split("\\.");
        int length = Math.max(thisParts.length, svParts.length);
        for(int i = 0; i < length; i++) {
            int thisPart = i < thisParts.length ?
                    Integer.parseInt(thisParts[i]) : 0;
            int svPart = i < svParts.length ?
                    Integer.parseInt(svParts[i]) : 0;
            if(thisPart < svPart)
                return -1;
            if(thisPart > svPart)
                return 1;
        }
        return 0;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o == null) {
            return false;
        }

        if (this.getClass() != o.getClass()) {
            return false;
        }

        return this.compareTo((SemanticVersioner) o) == 0;
    }
}
